#include "KinectSDKRangeToWorldFilter.h"



//----------------------------------------------------------------------------
template<class TRangeImage, class TFlagImage, class TWorldCoordImage>
KinectRangeToWorldFilter<TRangeImage,TFlagImage,TWorldCoordImage>
::KinectRangeToWorldFilter()
{
	m_XToZ = 0;
	m_YToZ = 0;
}


//----------------------------------------------------------------------------
template<class TRangeImage, class TFlagImage, class TWorldCoordImage>
KinectRangeToWorldFilter<TRangeImage,TFlagImage,TWorldCoordImage>
::~KinectRangeToWorldFilter()
{
}


//----------------------------------------------------------------------------
template<class TRangeImage, class TFlagImage, class TWorldCoordImage>
void
KinectRangeToWorldFilter<TRangeImage,TFlagImage,TWorldCoordImage>
::SetConversionParameters(float XToZ, float YToZ)
{
	m_XToZ = XToZ;
	m_YToZ = YToZ;
}


//----------------------------------------------------------------------------
template<class TRangeImage, class TFlagImage, class TWorldCoordImage>
void
KinectRangeToWorldFilter<TRangeImage,TFlagImage,TWorldCoordImage>
::GetConversionParameters(float *XToZ, float *YToZ)
{
	*XToZ = m_XToZ;
	*YToZ = m_YToZ;
}


//----------------------------------------------------------------------------
template<class TRangeImage, class TFlagImage, class TWorldCoordImage>
void
KinectRangeToWorldFilter<TRangeImage,TFlagImage,TWorldCoordImage>
::BeforeThreadedGenerateData()
{
	// Sanity check
	if ( fabs(m_XToZ) < 1e-3 || fabs(m_YToZ) < 1e-3 )
		itkExceptionMacro(<<"Invalid conversion parameters detected!");
}


//----------------------------------------------------------------------------
template<class TRangeImage, class TFlagImage, class TWorldCoordImage>
void
KinectRangeToWorldFilter<TRangeImage,TFlagImage,TWorldCoordImage>
::ThreadedGenerateData(const OutputImageRegionType& OutputRegionForThread,int threadId)
{
#if 0
	// Set up iterators
	typedef itk::ImageRegionConstIterator<InputImageType> RangeImageIteratorType;
	typedef itk::ImageRegionIterator<OutputImageType> WorldCoordIteratorType;
	RangeImageIteratorType itRange(GetInput(), OutputRegionForThread);
	WorldCoordIteratorType itWC(GetOutput(), OutputRegionForThread);

	//InputImageType::SizeType Resolution = GetInput()->GetRequestedRegion().GetSize();

	float Resolution[2];
	Resolution[0] = 640.f;
	Resolution[1] = 480.f;

	// Convert
	for ( itRange.GoToBegin(),itWC.GoToBegin(); !itRange.IsAtEnd(); ++itRange,++itWC )
	{
		// Get index
		InputImageType::IndexType Index = itWC.GetIndex();

		float fNormalizedX = (Index[0] / Resolution[0] - 0.5);
		itWC.Value()[0] = (float)(fNormalizedX * itRange.Value() * m_XToZ);

		float fNormalizedY = (0.5 - Index[1] / Resolution[1]);
		itWC.Value()[1] = (float)(fNormalizedY * itRange.Value() * m_YToZ);

		itWC.Value()[2] = itRange.Value();

		/*XnPoint3D Projective = xnCreatePoint3D(Index[0], Index[1], itRange.Value());
		XnPoint3D RealWorld;
		m_DepthGenerator->ConvertProjectiveToRealWorld(1,&Projective,&RealWorld);

		if ( fabs(itWC.Value()[0]-RealWorld.X) > 1e-3 )
			std::cout << "Mismatch in x-coord detected: " << itWC.Value()[0] << " vs. " << RealWorld.X << std::endl;
		if ( fabs(itWC.Value()[1]-RealWorld.Y) > 1e-3 )
			std::cout << "Mismatch in y-coord detected: " << itWC.Value()[1] << " vs. " << RealWorld.Y << std::endl;
		if ( fabs(itWC.Value()[2]-RealWorld.Z) > 1e-3 )
			std::cout << "Mismatch in z-coord detected: " << itWC.Value()[2] << " vs. " << RealWorld.Z << std::endl;*/
	}
#else
	InputImageType::SizeType Resolution = GetInput()->GetRequestedRegion().GetSize();

	const InputImageType::PixelType *RangePtr = GetInput()->GetBufferPointer() + GetInput()->ComputeOffset(OutputRegionForThread.GetIndex());
	OutputImageType::PixelType *WCPtr = GetOutput()->GetBufferPointer() + GetOutput()->ComputeOffset(OutputRegionForThread.GetIndex());
	float invResX = 1.f / Resolution[0];
	float invResY = 1.f / Resolution[1];

	/*if ( threadId == 1 )
	{
		std::cout << "OutputRegion for thread: " << OutputRegionForThread << std::endl;
		std::cout << "InputOffset=" << GetInput()->ComputeOffset(OutputRegionForThread.GetIndex()) << std::endl;
		std::cout << "OutputOffset=" << GetOutput()->ComputeOffset(OutputRegionForThread.GetIndex()) << std::endl;
	}*/

	for ( int y = 0; y < OutputRegionForThread.GetSize()[1]; y++ )
	{
		for ( int x = 0; x < OutputRegionForThread.GetSize()[0]; x++ )
		{
			// Get the range value
			InputImageType::PixelType Range = *RangePtr;

			// Comput x-coordinate
			//InputImageType::PixelType  fNormalizedX = (x / Resolution[0] - 0.5);
			InputImageType::PixelType  fNormalizedX = (x * invResX - 0.5);
			(*WCPtr)[0] = (InputImageType::PixelType )(fNormalizedX * Range * m_XToZ);

			// Compute y-coordinate
			//RangeImageType::PixelType  fNormalizedY = (0.5 - y / Resolution[1]);
			InputImageType::PixelType  fNormalizedY = (0.5 - y * invResY);
			(*WCPtr)[1] = (InputImageType::PixelType )(fNormalizedY * Range * m_YToZ);

			// Store z-component
			(*WCPtr)[2] = Range;

			// Update pointers
			RangePtr++;
			WCPtr++;
		}
	}
#endif

}


//----------------------------------------------------------------------------
template<class TRangeImage, class TFlagImage, class TWorldCoordImage>
void
KinectRangeToWorldFilter<TRangeImage,TFlagImage,TWorldCoordImage>
::AfterThreadedGenerateData()
{
}
